/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_invite.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

int cmd_invite (client_t *cl, int pargc, char *pargv[])
{
	char userhost[USERHOSTLEN];	/* nick!user@host of client */

	index_t *index;
	client_t *target;
	channel_t *chan;
	char *name, *nick;

	if (pargc < 2) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "INVITE"); return 1; }

	nick = pargv[1];
	name = pargv[2];

	/* client */
	target = ui.client_find(nick);
	if (!target) { ui.cl_send_numeric(cl, ERR_NOSUCHNICK, nick); return 1; }

	/* channel */
	chan = ui.channel_find(name);
	if (!chan) { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); return 1; }

	/* check target is not already on the channel */
	index = ui.client_find_chan(target,chan);
	if (index) return 1;

	/* check client is on channel */
	index = ui.client_find_chan(cl,chan);
	if (!index) { ui.cl_send_numeric(cl, ERR_NOTONCHANNEL, chan->name);  return 1; }

	/* check client is an chan-op */
	index = index->cref;
	if (!bit_tst(index,CF_OP)) { ui.cl_send_numeric(cl, ERR_CHANOPRIVSNEEDED, chan->name); return 1; }

	/* check target is not already invited */
	index = ui.client_find_invite(target,chan);
	if (index) return 1;

	/* add invite to list */
	index = sys_malloc(sizeof(*index));
	index->item = chan;
	ll_add_end(&target->invite_index,index);

	/* generate the userhost */
	ui.client_userhost(cl,userhost,true);

	/* send invite */
	ui.cl_send_numeric(cl, RPL_INVITING, target->name, chan->name);
	ui.cl_send_fmt(target, ":%s INVITE %s :%s", userhost, target->name, chan->name);

	return 0;
}
