/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_join.c,v 1.6 2002/01/19 01:04:24 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

static bool is_banned (channel_t *chan, char *userhost, char *userip)
{
	ban_t *ban;

	for (ban = (ban_t *) chan->bans.item_start; ban; ban = ban->next)
	{
		/* host or ip based ban */
		if (str_match(userip,   ban->str, false)) return true;
		if (str_match(userhost,	ban->str, false)) return true;
	}
	return false;
}

int cmd_join (client_t *cl, int pargc, char *pargv[])
{
	bool priv;
	int i = 1, j = 0;
	channel_t *chan;
	char *name, *key;
	char *joined [MAX_ARGS];	/* list of channels joined */
	char userhost[USERHOSTLEN];	/* nick!user@host of client */
	char userip  [USERIPLEN];	/* nick!user@ip   of client */

	if (pargc < 1) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "JOIN"); return 1; }
	userhost[0] = '\0';

	/* FIXME: make it an option for opers to join */
	priv = bit_tst(cl, FL_PRIV);

	while (name = pargv[i++])
	{
		/* extract valid channel name */
		if (name[0] != '#')
			{ ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); continue; }
		else
		{	
			char chname[CHANLEN];

			chname[0] = name[0];
			str_extract_valid(chname + 1, name + 1, CHANLEN - 1, ui.valid_chan_char);

			/* is there a key sent with the JOIN request? */
			key = NULL;
			if (pargv[i] && pargv[i][0] != '#') key = pargv[i++];

			/* sanity check channel name */
			if (chname[1] == '\0') { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); continue; }
			strcpy(name,chname);
		}

		/* max channel limit (opers & services exempt) */
		if (!priv && ui.Options->max_channels && cl->chan_index.nitems >= ui.Options->max_channels)
		{
			ui.cl_send_numeric(cl, ERR_TOOMANYCHANNELS, cl->name);
			break;
		}

		/* generate the userhosts (if required) */
		if (userhost[0] == '\0')
		{
			ui.client_userhost(cl,userhost,true);
			ui.client_userip(cl,userip,true);
		}

		/* check permissions to join channel (invite, keyed etc) */
		chan = ui.channel_find(name);
		if (chan)
		{
			index_t *index;

			/* check not already on channel */
			index = ui.client_find_chan(cl,chan);
			if (index) continue;

			if (!priv)
			{
				/* check bans */
				if (is_banned(chan,userhost,userip))
					{ ui.cl_send_numeric(cl, ERR_BANNEDFROMCHAN, chan->name); continue; }

				/* check key */
				if (bit_tst(chan, CM_KEY) && (!key || strcmp(chan->key, key)))
					{ ui.cl_send_numeric(cl, ERR_BADCHANNELKEY, chan->name); continue; }

				/* check limit */
				if (bit_tst(chan, CM_LIMIT) && (chan->members.nitems >= chan->limit))
					{ ui.cl_send_numeric(cl, ERR_CHANNELISFULL, chan->name); continue; }
			}

		#ifdef HAVE_SSL
			/* check ssl */
			if (bit_tst(chan, CM_SSLONLY) && !bit_tst(cl,FL_SSL))
				{ ui.cl_send_numeric(cl, ERR_SSLONLYCHAN, chan->name); continue; }
		#endif /* HAVE_SSL */

			/* check invite (keep as last check otherwise index removal is out of place) */
			if (bit_tst(chan, CM_INVITEONLY))
			{
				/* check if we have an invite for this channel */
				index = ui.client_find_invite(cl,chan);
				if (index)
					/* del invite from list */
					ui.invite_remove(cl,index);
				else if (!priv)
					{ ui.cl_send_numeric(cl, ERR_INVITEONLYCHAN, chan->name); continue; }
			}
		}

		/* add client to the channel */
		chan = ui.channel_join(chan, name, cl, userhost);
		joined[j++] = name;

		/* send topic to client */
		if (chan->topic[0])
		{
			ui.cl_send_numeric(cl, RPL_TOPIC, chan->name, chan->topic);
			if (chan->tv_topic)
				ui.cl_send_numeric(cl, RPL_TOPICSTAMP,
					chan->name, chan->topic_user, chan->tv_topic);
		}

		/* send names to client */
		ui.cl_send_names(cl,chan,true);
 	}

	/* send NJOIN message to other servers */
	ui.cl_send_njoin(cl, joined, j);

	return 0;
}	
