/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_list.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

#if 0
LIST #barrysworld
:hub.uk.quakenet.org 321 Bod Channel :Users  Name
:hub.uk.quakenet.org 322 Bod #barrysworld 120 :New Beta Q3 on FTP
:hub.uk.quakenet.org 323 Bod :End of /LIST
#endif

#if 0
:hub.uk.quakenet.org 334 Bod :Usage: /QUOTE LIST parameters
:hub.uk.quakenet.org 334 Bod :Where parameters is a space or comma seperated list of one or more of:
:hub.uk.quakenet.org 334 Bod : <max_users    ; Show all channels with less than max_users.
:hub.uk.quakenet.org 334 Bod : >min_users    ; Show all channels with more than min_users.
:hub.uk.quakenet.org 334 Bod : C<max_minutes ; Channels that exist less than max_minutes.
:hub.uk.quakenet.org 334 Bod : C>min_minutes ; Channels that exist more than min_minutes.
:hub.uk.quakenet.org 334 Bod : T<max_minutes ; Channels with a topic last set less than max_minutes ago.
:hub.uk.quakenet.org 334 Bod : T>min_minutes ; Channels with a topic last set more than min_minutes ago.
:hub.uk.quakenet.org 334 Bod :Example: LIST <3,>1,C<10,T>0  ; 2 users, younger than 10 min., topic set.
#endif

int cmd_list (client_t *cl, int pargc, char *pargv[])
{
	channel_t *chan;
	char *arg;
	bool priv;
	int i = 1;

	ui.cl_send_numeric(cl, RPL_LISTSTART);
	arg = pargv[1];

	/* FIXME: make it an option for opers to see secret channels */
	priv = bit_tst(cl, FL_PRIV);

	if (arg && arg[0] == '#')
	{
		/* list selected channels */
		while (arg = pargv[i++])
		{
			/* sanity check channel name */
			if (arg[0] != '#') { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, arg); continue; }

			/* secret or invalid channel check */
			chan = ui.channel_find(arg);
			if (!chan || (bit_tst(chan, CM_SECRET) && !priv))
				{ ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, arg); continue; }

			/* send list data */
			ui.cl_send_numeric(cl, RPL_LIST, chan->name, chan->members.nitems, chan->topic);
		}
	}
	else
	{
		/* list channels that match parameters */
		list_options_t opts = { -1, -1, -1, -1, -1, -1 };
		int *pmin = NULL, *pmax = NULL;

		while (arg = pargv[i++]) while (*arg)
		{
			switch (*arg)
			{
			case '<':
				if (!pmax) pmax = &opts.max_users;
				*pmax = atoi(arg + 1);
				pmin = pmax = NULL;
				*arg = '\0';
				break;

			case '>':
				if (!pmin) pmin = &opts.min_users;
				*pmin = atoi(arg + 1);
				pmin = pmax = NULL;
				*arg = '\0';
				break;

			case 'c':
			case 'C':
				pmin = &opts.min_chan;
				pmax = &opts.max_chan;
				arg++;
				break;

			case 't':
			case 'T':
				pmin = &opts.min_topic;
				pmax = &opts.max_topic;
				arg++;
				break;

			default:
				arg++;
				break;
			}
		}
		ui.channel_list(cl,priv,&opts);
	}

	ui.cl_send_numeric(cl, RPL_LISTEND);
	return 0;
}
