/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_mode.c,v 1.4 2002/01/29 18:39:38 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

int cmd_mode (client_t *cl, int pargc, char *pargv[])
{
	channel_t *chan;
	char *name;
	bool priv;

	if (pargc < 1) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "MODE"); return 1; }
	name = pargv[1];

	/* FIXME: make it an option for opers to change other user modes */
	priv = bit_tst(cl, FL_PRIV);

	/* check channel / client exists */
	if (name[0] == '#')
	{
		index_t *index;

		/* channel */
		chan = ui.channel_find(name);
		if (!chan) { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); return 1; }

		index = ui.client_find_chan(cl,chan);
		if (index) index = index->cref;

		/* return channel modes */
		if (pargc == 1)
		{
			char *key = "", buf[BUFSIZE];
			u_int16_t limit = 0;

			/* is the channel secret? */
			if (bit_tst(chan,CM_SECRET) && !index && !priv)
				{ ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); return 1; }

			/* send (visible) mode string */
			if (index || priv) { key = chan->key; limit = chan->limit; }
			ui.channel_mode_str(buf, '+', chan->flags, key, limit);
			ui.cl_send_numeric(cl, RPL_CHANNELMODEIS, chan->name, buf);
		}
		/* modify channel mode */
		else
		{
			if (!priv)
			{
				/* check client is on channel */
				if (!index) { ui.cl_send_numeric(cl, ERR_NOTONCHANNEL, chan->name); return 1; }
				/* check the client has enough priviledges to change modes */
				priv = bit_tst(index,CF_OP);
			}

			/* parse the channel modes */
			ui.channel_parse_mode_str(cl,chan,pargv + 2,priv);
		}
	}
	/* client flags */
	else
	{
		client_t *target;

		/* confirm we can change this client's flags */
		if (priv)
		{
			target = ui.client_find(name);
			if (!target) { ui.cl_send_numeric(cl, ERR_NOSUCHNICK, name); return 1; }
		}
		else if ( stricmp(cl->name, name) )
			{ ui.cl_send_numeric(cl, ERR_USERSDONTMATCH); return 1; }
		else
			target = cl;

		/* return user modes */
		if (pargc == 1)
		{
			char buf[BUFSIZE];

			ui.client_mode_str(buf, '+', target->flags);
			ui.cl_send_fmt(cl,":%s %.3d %s %s", ui.Srv->name, (int) RPL_UMODEIS, target->name, buf);
		}
		else
			ui.client_parse_mode_str(cl,target,pargv[2],priv);
	}

	return 0;
}
