/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_nick.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

static bool is_nick_reserved (const char *nick)
{
	int32_t i;
	array_t *array = &ui.Options->rsv_nicks;
	char **raw = (char **) array->list;

	for (i = 0; i < array->nitems; i++) if ( !stricmp(raw[i], nick) ) return true;

	return false;
}

int cmd_nick (client_t *cl, int pargc, char *pargv[])
{
	char *nick;
	char buf[sizeof(cl->name)];

	if (pargc < 1) goto bail;
	nick = pargv[1];

	/* extract valid nick name */
	if (isdigit(nick[0])) goto bail;	/* 1st char must not be a digit */
	str_extract_valid(buf, nick, sizeof(buf), ui.valid_nick_char);
	if (buf[0] == '\0') goto bail;
	nick = buf;

	/* check nick does not already exist */
	if ( ui.client_find(nick) ) { ui.cl_send_numeric(cl, ERR_NICKNAMEINUSE, nick); return 1; }

	/* check nick is not reserved or a service nick */
	if ( is_nick_reserved(nick) || buf[1] == '\0') { ui.cl_send_numeric(cl, ERR_SERVICENICK, nick); return 1; }

	/* send cookie PING as required */
	ui.client_cookie(cl);

	bit_set(cl,FL_GOTNICK);

	/* if nick is same ignore */
	if ( stricmp(cl->name, nick) )
	{
		/* change the nick */
		ui.client_set_nick(cl,nick);
	}
	return 0;
bail:
	ui.cl_send_numeric(cl, ERR_NONICKNAMEGIVEN);
	return 1;
}
