/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_privmsg.c,v 1.4 2002/01/27 22:34:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

int send_msg (client_t *cl, char *name, char *text, bool privmsg)
{
	char userhost[USERHOSTLEN];	/* nick!user@host of client */

	/* check channel / client exists */
	if (name[0] == '#' || name[0] == '@')
	{
		index_t *index;
		channel_t *chan;
		u_int32_t filter = 0;

		if (name[0] == '@')
		{
			if (*++name != '#') { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); return 1; }
			filter = CF_OP;
		}

		/* channel */
		chan = ui.channel_find(name);
		if (!chan) { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); return 1; }

		/* check client is on channel */
		index = ui.client_find_chan(cl,chan);
		if (index) index = index->cref;

		/* if no extern messages, check client is on channel */
		if (bit_tst(chan, CM_NOPRIVMSGS) && !index)
			{ ui.cl_send_numeric(cl, ERR_CANNOTSENDTOCHAN, chan->name); return 1; }

		/* if moderated check user permissions */
		if (privmsg && bit_tst(chan, CM_MODERATED) && (!index || !bit_tst(index, CF_OP | CF_VOICE)) ) return 1;

		/* generate the userhost */
		ui.client_userhost(cl,userhost,true);

		/* send to all members (except client) */
		ui.chan_send_fmt(chan, cl, 0, filter, ":%s %s %s :%s",
			userhost, privmsg ? "PRIVMSG" : "NOTICE", chan->name, text);
	}
	else
	{
		/* client */
		client_t *target;
		char *host, *uhost;

		/* check if the target is a 'user@host' */
		if (host = strchr(name,'@')) *host++ = '\0';

		/* generate the userhost */
		ui.client_userhost(cl,userhost,true);
		uhost = strchr(userhost,'@');

		/* find the target */
		target = ui.client_find(name);
		if (!target || (host && uhost && stricmp(uhost+1,host)))
			{ ui.cl_send_numeric(cl, ERR_NOSUCHNICK, name); return 1; }

		/* is the target client away? */
		if (privmsg && target->u.away[0]) ui.cl_send_numeric(cl, RPL_AWAY, target->name, target->u.away);

		/* send to target client */
		ui.cl_send_fmt(target, ":%s %s %s :%s",
			userhost, privmsg ? "PRIVMSG" : "NOTICE", target->name, text);
	}

	return 0;
}

int cmd_privmsg (client_t *cl, int pargc, char *pargv[])
{
	if (pargc >= 2) return send_msg(cl, pargv[1], pargv[2], true);

	if (pargc == 1)
		ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "PRIVMSG");
	else
		ui.cl_send_numeric(cl, ERR_NOTEXTTOSEND);

	return 1;
}
