/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_topic.c,v 1.4 2002/01/27 22:34:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

int cmd_topic (client_t *cl, int pargc, char *pargv[])
{
	bool priv;
	char *name;
	index_t *index;
	channel_t *chan;

	if (pargc < 1) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "TOPIC"); return 1; }
	name = pargv[1];

	/* find channel */
	chan = ui.channel_find(name);
	if (!chan) { ui.cl_send_numeric(cl, ERR_NOSUCHCHANNEL, name); return 1; }

	/* display topic */
	if (pargc == 1 || pargv[2][0] == '\0')
	{
		if (chan->topic[0])
			ui.cl_send_numeric(cl, RPL_TOPIC, chan->name, chan->topic);
		else
			ui.cl_send_numeric(cl, RPL_NOTOPIC, chan->name);
		if (chan->tv_topic)
			ui.cl_send_numeric(cl, RPL_TOPICSTAMP,
				chan->name, chan->topic_user, chan->tv_topic);
		return 0;
	}

	/* FIXME: make it an option for opers to change topics */
	priv = bit_tst(cl, FL_PRIV);

	/* check client is on channel */
	index = ui.client_find_chan(cl,chan);
	if (index) index = index->cref;

	/* check client can set topic */
	if (!priv && bit_tst(chan,CM_TOPICLIMIT))
	{
		if (!index) { ui.cl_send_numeric(cl, ERR_NOTONCHANNEL, chan->name); return 1; }
		if (!bit_tst(index,CF_OP)) { ui.cl_send_numeric(cl, ERR_CHANOPRIVSNEEDED, chan->name); return 1; }
	}

	/* copy the topic */
	str_copy(chan->topic, pargv[2], sizeof(chan->topic));
	str_copy(chan->topic_user, cl->name, sizeof(chan->topic_user));
	chan->tv_topic = ui.Now->tv_sec;

	/* send topic to client */
	if (chan->topic[0])
	{
		char userhost[USERHOSTLEN];	/* nick!user@host of client */

		/* generate the userhost */
		ui.client_userhost(cl,userhost,true);

		ui.chan_send_fmt(chan, NULL, FL_BROADCAST | FL_LOCAL, 0,
			":%s TOPIC %s :%s", userhost, chan->name, chan->topic);
	}

	return 0;
}
