/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_who.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

static void who_user (client_t *cl, client_t *target, index_t *index, char *mask, bool priv, bool show)
{
	char userhost[USERHOSTLEN];	/* nick!user@host of client */
	char buf[32], *str;

	/* if invisible, check client can see target */
	if (bit_tst(target,FL_INVISIBLE) && !show) return;

	str = buf;
	*str++ = target->u.away[0] ? 'G' : 'H';

	if (priv) *str++ = '*';
	if (index)
	{
		if ( bit_tst(index,CF_OP) )		 *str++ = '@';
		if ( bit_tst(index,CF_VOICE) )	 *str++ = '+';
	}
	*str = '\0';

	/* generate the userhost */
	ui.client_userhost(target, userhost, priv ? false : true);
	str = strchr(userhost,'@') + 1;

	ui.cl_send_numeric(cl, RPL_WHOREPLY, index ? mask : "*",
		target->user, str, ui.Srv->name,
		target->name, buf, 0, target->info);
}

int cmd_who (client_t *cl, int pargc, char *pargv[])
{
	int i = 1;
	char *mask;
	channel_t *chan;

	/* check client can see 'hidden' users */
	bool priv = bit_tst(cl, FL_PRIV);

	if (pargc < 1) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "WHO"); return 1; }

	/* WHO was invented to test the sanity of decent coders */

	while (mask = pargv[i++])
	{
		if (mask[0] == '#')
		{
			chan = ui.channel_find(mask);
			if (chan)
			{
				index_t *index;
				/* check client can see 'hidden' users */
				bool show = priv || ui.client_find_chan(cl,chan);

				/* if secret, check client can see channel */
				if (show || !bit_tst(chan, CM_SECRET))
					for (index = (index_t *) chan->members.item_start; index; index = index->next)
				{
					if (bit_tst(index,FL_ZOMBIE)) continue;
					who_user(cl, (client_t *) index->item, index, mask, priv, show);
				}
			}
		}
		else
		{
			client_t *target = ui.client_find(mask);
			if (target) who_user(cl, target, NULL, mask, priv, priv);
		}
		ui.cl_send_numeric(cl, RPL_ENDOFWHO, mask);
	}
	return 0;
}
