/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_whois.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

int list_channels (const char *dst, size_t dst_len, client_t *cl, client_t *target, bool hide)
{
	int rem = dst_len, len = -1;
	index_t *index, *cl_idx;
	char *str = (char *) dst;
	channel_t *chan;

	index = (index_t *) target->chan_index.item_start;
	if (!index) goto done;

	for (*str = '\0'; index; index = index->next)
	{
		chan = (channel_t *) index->item;

		/* hide secret channels the target is on, unless the client is in the channel */
		cl_idx = ui.client_find_chan(cl,chan);
		if (hide && bit_tst(chan,CM_SECRET) && !cl_idx) continue;

		len = strlen(chan->name);
		if (len > (rem - 2)) goto done;

		/* write users channel flags */
		cl_idx = ui.client_find_chan(target,chan);
		if (!cl_idx) continue;
		cl_idx = cl_idx->cref;

		if ( bit_tst(cl_idx,CF_OP) )
		{
			*str++ = '@';
			rem--;
		}
		else if ( bit_tst(cl_idx,CF_VOICE) )
		{
			*str++ = '+';
			rem--;
		}

		/* write channel name */
		memcpy(str,chan->name,len);
		str += len;
		rem -= len;

		/* write space */
		*str++ = ' ';
		rem--;
	}
	if (str > dst) --str;
done:
	/* write terminator */
	*str   = '\0';

	return (str - dst);
}

int cmd_whois (client_t *cl, int pargc, char *pargv[])
{
	client_t *target;
	char userhost[USERHOSTLEN];	/* nick!user@host of client */
	char *name, *user, *host;
	char buf[BUFSIZE];
	bool hide;

	if (pargc < 1) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "WHOIS"); return 1; }
	name = pargv[1];

	/* client */
	target = ui.client_find(name);
	if (!target) { ui.cl_send_numeric(cl, ERR_NOSUCHNICK, name); return 1; }

	/* generate the userhost */
	ui.client_userhost(target, userhost, bit_tst(cl,FL_OPERATOR) ? false : true);
	user = strchr(userhost,'!'); *user++ = '\0';
	host = strchr(user,'@'); *host++ = '\0';

	/* whois begin */
	ui.cl_send_numeric(cl, RPL_WHOISUSER, target->name, user, host, target->info);

	/* show channel list */
	hide = (target == cl || bit_tst(cl, FL_PRIV)) ? false : true;

	list_channels(buf,sizeof(buf),cl,target,hide);

	if (buf[0]) ui.cl_send_numeric(cl, RPL_WHOISCHANNELS, target->name, buf);

	/* is the target an oper */
	if (bit_tst(target,FL_OPERATOR)) ui.cl_send_numeric(cl, RPL_WHOISOPERATOR, target->name);
	/* away message */
	if (target->u.away[0]) ui.cl_send_numeric(cl, RPL_AWAY, target->name, target->u.away);
	/* idle time */
	ui.client_idle_time(cl,target);
	/* whois done */
	ui.cl_send_numeric(cl, RPL_ENDOFWHOIS, target->name);

	return 0;
}
