/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: plugin.c,v 1.6 2002/01/19 00:01:36 zarjazz Exp $
*/

#include <plugin.h>

#ifdef WINVER
  #pragma warning(disable : 4090)	// const
#endif

/* ================ */

cl_ui_t ui;

/* Command List */
static cmd_entry_t cmd_entry_list[] = 
{
	/* Message Commands */
	{ "PRIVMSG",	cmd_privmsg,	CMD_RDCHAN | CMD_ISREG | CMD_IDLE },
	{ "NOTICE",		cmd_notice,		CMD_ISREG | CMD_IDLE },
	/* Connection Commands */
	{ "PASS",		cmd_pass,		0 },
	{ "NICK",		cmd_nick,		0 },
	{ "USER",		cmd_user,		0 },
	{ "QUIT",		cmd_quit,		0 },
	/* Channel Commands */
	{ "JOIN",		cmd_join,		CMD_WRCHAN | CMD_ISREG },
	{ "PART",		cmd_part,		CMD_WRCHAN | CMD_ISREG }, /* FIXME: Write lock */
	{ "MODE",		cmd_mode,		CMD_RDCHAN | CMD_ISREG | CMD_IDLE },
	{ "TOPIC",		cmd_topic,		CMD_RDCHAN | CMD_ISREG },
	{ "NAMES",		cmd_names,		CMD_RDCHAN | CMD_ISREG },
	{ "LIST",		cmd_list,		CMD_RDCHAN | CMD_ISREG },
	{ "INVITE",		cmd_invite,		CMD_RDCHAN | CMD_ISREG },
	{ "KICK",		cmd_kick,		CMD_RDCHAN | CMD_ISREG },
	/* Server Commands */
	{ "VERSION",	cmd_version,	0 },
	{ "STATS",		cmd_stats,		CMD_ISREG },
	{ "LINKS",		cmd_links,		CMD_ISREG },
	{ "TIME",		cmd_time,		CMD_ISREG },
	{ "ADMIN",		cmd_admin,		0 },
	{ "MOTD",		cmd_motd,		0 },
	{ "INFO",		cmd_info,		0 },
	/* Query Commands */
	{ "WHO",		cmd_who,		CMD_RDCHAN | CMD_ISREG },
	{ "WHOIS",		cmd_whois,		CMD_RDCHAN | CMD_ISREG },
	{ "WHOWAS",		NULL,			CMD_ISREG },
	/* Misc Commands */
	{ "PING",		cmd_ping,		CMD_ISREG },
	{ "PONG",		cmd_pong,		0 },
	/* Other Commands */
	{ "AWAY",		cmd_away,		CMD_ISREG },
	{ "LUSERS",		cmd_lusers,		CMD_ISREG },
	{ "USERHOST",	cmd_userhost,	CMD_ISREG },
	{ "USERIP",		cmd_userip,		CMD_ISREG },
	{ "ISON",		cmd_ison,		CMD_ISREG },
	{ "HELP",		NULL,			CMD_ISREG },
	/* Terminator */
	{	NULL, NULL, 0 }
};

/*
================
query_remote
================
*/

int query_remote (client_t *sv, client_t *cl, char *str)
{
	if (!sv && str)
	{
		client_t *remote = ui.server_match(str);

		if (remote) return 2;

		ui.cl_send_numeric(cl, ERR_NOSUCHSERVER, str);
		return 1;
	}
	return 0;
}

/*
================
init_client_interface
----------------
plugin initialization
================
*/

cmd_entry_t *init_client_interface (cl_ui_t *srv_ui)
{
	ui = *srv_ui;
	return cmd_entry_list;
}



