/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: cmd_oper.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All cmd_* functions are called with the client read lock set *
 ****************************************************************/

static u_int32_t check_oper (client_t *cl, const char *nick, const char *pass)
{
	block_t *block = NULL;
	block_t *root = *(ui.Config);
	block_arg_t *blk_arg;
	char *blk_val;

	char userhost[USERHOSTLEN];	/* nick!user@host of client */
	char userip  [USERIPLEN];	/* nick!user@ip   of client */

	/* generate the userhost */
	ui.client_userhost(cl,userhost,true);
	ui.client_userip(cl,userip,true);

	/* step through Oper blocks */
	while ( block = blocks_by_name(root,block,"Oper") )
	{
		bool found = false;
		u_int32_t flags;

		/* match nick */
		if ( stricmp(nick, block->block_value) ) continue;

		/* match host or ip mask */
		blk_arg = NULL;
		while ( blk_arg = arguments_by_name(block,blk_arg,"Mask") )
		{
			if (str_match(userip,   blk_arg->arg_value, false) ||
				str_match(userhost,	blk_arg->arg_value, false))
					{ found = true; break; }
		}

		if (!found) { ui.cl_send_numeric(cl, ERR_NOOPERHOST); return false; }

		/* match password */
		blk_val = argument_by_name(block,"Password");
		if ( strcmp(pass, blk_val) ) { ui.cl_send_numeric(cl, ERR_PASSWDMISMATCH); return false; }

		/* get the oper automode */
		flags = 0;
		blk_val = argument_by_name(block,"AutoMode");
		if (blk_val)
		{
			char *p;
			for (p = blk_val; *p; p++) flags |= ui.user_mode(*p);
		}

		/* hurrah, we have a match! :) */
		return flags | FL_OPERATOR;
	}

	return 0;
}

int cmd_oper (client_t *cl, int pargc, char *pargv[])
{
	char userhost[USERHOSTLEN];	/* nick!user@host of client */
	char *nick, *pass;
	u_int32_t flags;

	if (pargc < 2) { ui.cl_send_numeric(cl, ERR_NEEDMOREPARAMS, "OPER"); return 1; }

	nick = pargv[1];
	pass = pargv[2];

	if (bit_tst(cl,FL_OPERATOR)) { ui.cl_send_numeric(cl, ERR_ALREADYREGISTRED); return 1; }

	/* generate the userhost */
	ui.client_userhost(cl,userhost,true);

	/* check valid nick & pass */
	if (flags = check_oper(cl,nick,pass))
	{
		char buf[BUFSIZE];

		/* actual mode change */
		flags &= ~FL_SERVICE;	/* yeah right.. */
		flags &= ~cl->flags;
		cl->flags |= flags;


		/* send mode string & notification */
		ui.client_mode_str(buf,'+',flags);
		ui.cl_send_fmt(cl, ":%s MODE %s :%s", userhost, cl->name, buf);

		ui.cl_send_numeric(cl, RPL_YOUREOPER);

		/* notify all '+s' users about new operator */
		ui.server_notice("New Operator", userhost);

		/* user stats */
		ui.UserStats->N_opers++;
	}
	else
		ui.all_send_fmt(FL_WALLOPS, ":%s WALLOPS :Failed OPER attempt by %s", ui.Srv->name, userhost);

	return 0;
}

