/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: msg_msg.c,v 1.4 2002/01/29 18:39:38 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All msg_* functions are called with the client read lock set *
 ****************************************************************/

int msg_msg (client_t *sv, int pargc, char *pargv[])
{
	char *str, *msg;
	int len;

	if (pargc < 2) { ui.sv_send_numeric(sv, ERR_NEEDMOREPARAMS, "MSG"); return 1; }
	str = pargv[1];
	msg = pargv[2];

	/* remove spurious white spaces */
	len = strlen(msg);
	while (len > 0 && isspace(msg[len - 1])) msg[--len] = '\0';

	/* append terminating newline */
	msg[len++] = '\r';
	msg[len++] = '\n';

	/* what is the destination of the message? */
	if (isdigit(*str))
	{
		/* server */
		u_int16_t id = atoi(str);
		client_t *dst = ui.server_find_id(id);

		if (ui.Srv == dst)
		{
			/* FIXME: the message is for this server */
			ui.server_notice(str,NULL);
		}
		else
			ui.sv_send_raw(dst, pargv[2], len);
	}
	else if (*str == '#')
	{
		/* channel */
		channel_t *chan = ui.channel_find(str);
		index_t *index;

		if (!chan) return 1;

		/* send text to all LOCAL channel members */
		for (index = (index_t *) chan->members.item_start; index; index = index->next)
		{
			client_t *cl;
			cl = (client_t *) index->item;
			if (is_local(cl)) ui.sv_send_raw(cl, pargv[2], len);
		}
	}
	else
	{
		client_t *cl = ui.client_find(str);
		if (!cl) return 1;

		ui.sv_send_raw(cl,pargv[2],len);
	}

	return 0;
}
