/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: msg_server.c,v 1.6 2002/01/27 22:34:36 zarjazz Exp $
*/

#define __USE_SERVER_STRUCT
#include <plugin.h>

/****************************************************************
 * All msg_* functions are called with the client read lock set *
 ****************************************************************/

int msg_server (client_t *sv, int pargc, char *pargv[])
{
	char *name;
	client_t *x;

	if (pargc < 3) { ui.sv_send_numeric(sv, ERR_NEEDMOREPARAMS, "SERVER"); return 1; }
	name = pargv[1];

	/* is this is an introduction from a remote server? */
	if (pargv[0])
	{
		client_t *new_sv;
		char *name = pargv[1];
		u_int16_t id = atoi(pargv[3]);

		/* do we already know about this server? */
		if (new_sv = ui.server_find_id(id))
		{
			sys_dprintf(1,"server already known: %s (%hu)\n", new_sv->name, id);
			return 0;
		}
		/* FIXME: server name clash */
		new_sv = ui.server_find(name);
#if 0
		if (new_sv->node.id != id)
		{
			sys_dprintf(1, "illegal introduction: %s (%hu)\n", new_sv->name, id);
			return -1;
		}
#endif
		/* create new server if required */
		if (!new_sv)
		{
			new_sv = ui.server_malloc(id);
			str_copy(new_sv->name, name, sizeof(new_sv->name));
			/* FIXME: write lock */
			ui.server_create(new_sv, false);
		}

		sys_dprintf(1,"Server Introduction: %s (%hu)\n",name,id);
	}
	/* this must be a connect to the local server then */
	else
	{
		int hopcount;
		u_int16_t token;
		char info[sizeof(sv->info)];

		if (bit_tst(sv,FL_REGISTERED))
			{ ui.sv_send_numeric(sv, ERR_ALREADYREGISTRED); return 1; }

		hopcount = atoi(pargv[2]);
		token	 = atoi(pargv[3]);

		/* info */
		if (pargv[4])
		{
			str_copy(info, str_strip(pargv[4]), sizeof(info));
			if (info[0] == '\0') strcpy(info,"No Info");
		}

		/* parsed server sanity checks */
		x = ui.server_find(name);
		if (!x || !bit_tst(x,FL_LOCAL))
			{ ui.sv_send_notice(sv, "illegal connect from", name); return -1; }
		if (bit_tst(x,FL_CONNECTED))
			{ ui.sv_send_notice(sv, "duplicate connect from", name); return -1; }
		/* check: password */
		if (strcmp(sv->u.pass,ui.server_password(x)))
			{ ui.sv_send_notice(sv, "bad password from", name); return -1; }
		/* check: token */
		if (!token || ui.server_find_id(token))
			{ ui.sv_send_notice(sv, "duplicate id from", name); return -1; }

		/* copy the accepted server details */
		ui.server_copy(x,sv,token);
		bit_set(x,FL_CONNECTED | FL_LOCAL);

		/* remove the temporary structure */
		bit_clr(sv,FL_CONNECTED);
		ui.server_exit(sv,"temporary");

		ui.server_notice("Server Accepted", x->name);

		/* if this server isn't known, start the sync */
		if (!bit_tst(x,FL_REGISTERED)) ui.sync_start(x,NULL,true);
	}

	return 0;
}
