/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: msg_sync.c,v 1.7 2002/01/29 18:39:38 zarjazz Exp $
*/

#include <plugin.h>

/****************************************************************
 * All msg_* functions are called with the client read lock set *
 ****************************************************************/

int msg_sync (client_t *sv, int pargc, char *pargv[])
{
	channel_t *chan;
	client_t *src;

	/* if we dont have a prefix assume its the server who sent this message */
	src = get_prefix(pargv[0]);
	if (!src) src = sv;

	/* check the source isn't already registered */
	if (bit_tst(src,FL_REGISTERED))
	{
		sys_dprintf(1,"%s already sync'd\n", src->name);
		return 1;
	}

	/* is this server requesting sync data? */
	if (!stricmp(pargv[1],"REQUEST"))
		ui.sync_start(src,pargv[0],false);
	/* is the sync complete? */
	else if (!strcmp(pargv[1],"DONE"))
	{
		ui.server_register(src);
		ui.server_notice("Sync Complete",src->name);
	}
	/* is this the channel mode details? */
	else if (strcmp(pargv[1],"NAMES"))
	{
		char *name;

		/* sanity check */
		if (pargc < 2) { ui.sv_send_numeric(sv, ERR_NEEDMOREPARAMS, "SYNC"); return 1; }

		/* create channel if required */
		name = pargv[1];
		chan = ui.channel_find(name);
		if (!chan) chan = ui.channel_create(name);

		ui.channel_parse_mode_str(src,chan,pargv + 2,true);
	}
	/* sync users on the channel */
	else
	{
		char *names;

		/* sanity check */
		if (pargc < 3) { ui.sv_send_numeric(sv, ERR_NEEDMOREPARAMS, "SYNC"); return 1; }

		/* channel *must* exist */
		chan = ui.channel_find(pargv[2]);
		if (!chan) return 1;
		
		/* loop over each name */
		names = str_strip(pargv[3]);
		while (*names)
		{
			char nick[sizeof(sv->name)];
			char userhost[USERHOSTLEN];	/* nick!user@host of client */
			unsigned flags;
			client_t *cl;

			/* extract the clients channel flags */
			for (flags = 0; true; names++)
			{
				if (*names == '@')
					flags |= CF_OP;
				else if (*names == '+')
					flags |= CF_VOICE;
				else
					break;
			}

			/* extract valid nick name */
			names = str_extract_valid(nick, names, sizeof(nick), ui.valid_nick_char);
			if (nick[0] == '\0') break;

			/* nick *must* exist */
			cl = ui.client_find(nick);
			if (!cl) continue;

			/* add client to the channel */
			ui.client_userhost(cl,userhost,true);
			ui.channel_join(chan, NULL, cl, userhost);

			/* send the client flags */
			if (flags)
			{
				char *args[6], buf[32], *ptr = buf;

				*ptr++ = '+';
				if (flags & CF_OP)		*ptr++ = 'o';
				if (flags & CF_VOICE)	*ptr++ = 'v';

				args[0] = buf;
				args[1] = nick;
				args[2] = nick;
				args[3] = NULL;

				ui.channel_parse_mode_str(src,chan,args,true);
			}
		}
	}

	return 0;
}
