/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: core_threads.h,v 1.3 2002/01/27 22:34:36 zarjazz Exp $
*/


#include <pthread.h>


#ifndef __CORE_THREADS_H
#define __CORE_THREADS_H

#ifdef __cplusplus
extern "C" {
#endif

/* Typedefs */

typedef pthread_t			thread_t;
typedef pthread_rwlock_t	rwlock_t;
typedef pthread_mutex_t		mutex_t;
typedef pthread_cond_t		cond_t;

/* Defines */
#define THREAD_FUNC(x)		void * x (void *arg)
#define THREAD_KILL(thread)	pthread_kill(&(thread), SIGTERM)
#define THREAD_JOIN(thread)	pthread_join((thread), NULL)
#define THREAD_CREATE(thread,func,arg)	pthread_create(&(thread), NULL, (func), (arg))

#define RWLOCK_INIT(rwlock)		pthread_rwlock_init(&(rwlock), NULL)
#define RWLOCK_DESTROY(rwlock)	pthread_rwlock_destroy(&(rwlock))

#ifdef _DEBUG_THREADS
  #define READ_LOCK(rwlock)	do { \
									printf("%s %d RDLK+: " #rwlock "\n", __FILE__, __LINE__); \
									pthread_rwlock_wrlock(&(rwlock)); \
								} while (0)
  #define READ_UNLOCK(rwlock)	do { \
									printf("%s %d RDLK-: " #rwlock "\n", __FILE__, __LINE__); \
									pthread_rwlock_unlock(&(rwlock)); \
								} while (0)
  #define WRITE_LOCK(rwlock)	do { \
									printf("%s %d WRLK+: " #rwlock "\n", __FILE__, __LINE__); \
									pthread_rwlock_wrlock(&(rwlock)); \
								} while (0)
  #define WRITE_UNLOCK(rwlock)	do { \
									printf("%s %d WRLK-: " #rwlock "\n", __FILE__, __LINE__); \
									pthread_rwlock_unlock(&(rwlock)); \
								} while (0)
#else
  #define READ_LOCK(rwlock)		pthread_rwlock_rdlock(&(rwlock))
  #define READ_UNLOCK(rwlock)	pthread_rwlock_unlock(&(rwlock))

  #define WRITE_LOCK(rwlock)	pthread_rwlock_wrlock(&(rwlock))
  #define WRITE_UNLOCK(rwlock)	pthread_rwlock_unlock(&(rwlock))
#endif /* _DEBUG */

#define READ_TRYLOCK(rwlock)	pthread_rwlock_tryrdlock(&(rwlock))
//#define WRITE_TRYLOCK(rwlock)	pthread_rwlock_tryrdlock(&(rwlock))

#define MUTEX_INIT(mutex)		pthread_mutex_init(&(mutex), NULL)
#define MUTEX_DESTROY(mutex)	pthread_mutex_destroy(&(mutex))
#define MUTEX_LOCK(mutex)		pthread_mutex_lock(&(mutex))
#define MUTEX_TRYLOCK(mutex)	pthread_mutex_trylock(&(mutex))
#define MUTEX_UNLOCK(mutex)		pthread_mutex_unlock(&(mutex))

#define COND_INIT(cond)			pthread_cond_init(&(cond),NULL)
#define COND_WAIT(cond,mutex)	pthread_cond_wait(&(cond),&(mutex))
#define COND_SIGNAL(cond)		pthread_cond_signal(&(cond))
#define COND_BROADCAST(cond)	pthread_cond_broadcast(&(cond))
#define COND_DESTROY(cond)		pthread_cond_destroy(&(cond))


#ifdef __cplusplus
}
#endif

#endif /* __CORE_THREADS_H */	 
