/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: ircd_channel.h,v 1.5 2002/01/29 18:39:38 zarjazz Exp $
*/

#ifndef __SRV_CHANNEL_H
#define __SRV_CHANNEL_H

#ifdef __cplusplus
extern "C" {
#endif


/* Typedefs */

typedef struct channel_s
{
	/* binary tree elements */
	struct channel_s *left;
	struct channel_s *right;
	struct channel_s *parent;
	/* linked list elements */
	struct channel_s *prev;
	struct channel_s *next;
	/* binary tree key */
	char *tree_key;
	/* index */
	u_int32_t	id;
	/* channel memeber lists */
	llist_t		members;
	llist_t		bans;
	/* status flags */
	u_int32_t	flags;
	/* time stamps */
	int32_t	tv_create;
	int32_t	tv_topic;
	int32_t	tv_modify;
	/* channel settings */
	u_int16_t	limit;
	char key	[KEYLEN];
	/* static strings */
	char name		[CHANLEN];
	char topic		[TOPICLEN];
	char topic_user	[NICKLEN];
} channel_t;

typedef struct ban_s
{
	/* linked list elements */
	struct ban_s *next;
	struct ban_s *prev;
	/* structure data */
	char str[BANLEN];
	char who[NICKLEN];
	int32_t when;
} ban_t;

typedef struct list_options_s
{
	int max_users, min_users;
	int max_chan, min_chan;
	int max_topic, min_topic;
} list_options_t;

/* Channel Exit */

#define CE_QUIT		0
#define CE_PART		1
#define CE_KICK		2

/* Channel Modes */

#define CM_SECRET			(1 << 0)	/* '+s' */
#define CM_MODERATED		(1 << 1)	/* '+m' */
#define CM_NOPRIVMSGS		(1 << 2)	/* '+n' */
#define CM_TOPICLIMIT		(1 << 3)	/* '+t' */
#define CM_INVITEONLY		(1 << 4)	/* '+i' */
#define CM_KEY				(1 << 5)	/* '+k' */
#define CM_LIMIT			(1 << 6)	/* '+l' */

#ifdef HAVE_SSL
	#define CM_SSLONLY		(1 << 7)	/* '+x' */
#endif /* HAVE_SSL */


/* Prototypes */
#ifndef __PLUGIN
	void	init_channels (void);

	void channel_read_lock		(void);
	void channel_read_unlock	(void);

	channel_t *	channel_validate(u_int32_t id);
	channel_t *	channel_find	(const char *name);
	channel_t *	channel_create	(const char *name);

	void	channel_part	(channel_t *chan, client_t *target, client_t *cl, const char *comment, int type);
	void	channel_exit	(index_t *cl_idx, client_t *cl, const char *comment, int type);
	void	channel_list	(client_t *cl, bool priv, list_options_t *opts);
	int		channel_names	(const char *dst, size_t dst_len, channel_t *chan, index_t **marker, bool show);

	channel_t *	channel_join		(channel_t *chan, char *name, client_t *cl, char *userhost);
	index_t *	channel_find_nick	(channel_t *chan, const char *nick);
	index_t *	client_find_invite	(client_t *cl, channel_t *chan);
	index_t *	client_find_chan	(client_t *cl, channel_t *chan);

	int channel_mode_str		(char *buf, char fl, u_int32_t flags, char *key, u_int16_t limit);
	void channel_parse_mode_str	(client_t *cl, channel_t *chan, char **args, bool priv);
#endif /* __PLUGIN */


#ifdef __cplusplus
}
#endif

#endif /* __SRV_CHANNEL_H */	 
