/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: ircd_client.h,v 1.10 2002/01/19 00:01:36 zarjazz Exp $
*/

#ifndef __SRV_CLIENT_H
#define __SRV_CLIENT_H

#ifdef __cplusplus
extern "C" {
#endif

/* Typedefs */
#if defined(__sun__)
	#define index_t v_index_t
#endif

typedef struct index_s
{
	/* linked list elements */
	struct index_s *next;
	struct index_s *prev;
	struct index_s *cref;
	/* status flags */
	u_int32_t	flags;
	/* index */
	void	*item;
} index_t;

typedef struct client_s
{
	/* binary tree elements */
	struct client_s *left;
	struct client_s *right;
	struct client_s *parent;
	/* linked list elements */
	struct client_s *prev;
	struct client_s *next;
	/* binary tree key */
	char *tree_key;
	/* index */
	u_int32_t	id;
	/* local server */
	union
	{
		struct client_s	*server;
		thread_t thread;
	} local;
	/* channel indexes */
	llist_t		chan_index;
	llist_t		invite_index;
	/* status flags */
	u_int32_t	flags;
	/* static strings */
	char name	[NICKLEN];
	char user	[USERLEN];
	char info	[INFOLEN];
	union
	{
		char away[INFOLEN];
		char pass[PASSLEN];
	} u;
	/* dynamic strings*/
	char *host;
	char *hostcrypt;
	char *domain;
#ifndef __PLUGIN
	/* data specific to local users and servers */
	sendq_t	sendq;
	/* status flags */
	u_int32_t	cookie;
	/* time stamps */
	int32_t tv_connect;
	int32_t tv_ping;
	int32_t tv_active;
	/* network data */
	listener_t	*listener;
	struct sockaddr_in sa;
	u_int16_t	sock;
	/* client / server specific data */
#ifdef HAVE_SSL
	SSL *ssl;
#endif /* HAVE_SSL */
#ifdef __USE_SERVER_STRUCT
	node_t node;
	bool die;
#endif /* __USE_SERVER_STRUCT */
#endif /* __PLUGIN */
} client_t;

/* Client Flags */
#define FL_RESOLVED			(1 << 0)
#define FL_REGISTERED		(1 << 1)
#define FL_GOTNICK			(1 << 2)
#define FL_ENCRYPTHOST		(1 << 3)
#define FL_PING_SENT		(1 << 4)
#define FL_BUSY				(1 << 5)
#define FL_LOCAL			(1 << 6)
#define FL_SYNCING			(1 << 7)
#define FL_SENDING			(1 << 8)

/* Server Flags */
#define FL_SERVER			(1 << 10)
#define FL_SERVICE			(1 << 11)
#define FL_CONNECTED		(1 << 12)

/* SSL */
#ifdef HAVE_SSL
#define FL_SSL				(1 << 15)
#endif /* HAVE_SSL */

/* Client UserMode Flags */
#define FL_OPERATOR			(1 << 16)
#define FL_INVISIBLE		(1 << 17)
#define FL_SNOTICE			(1 << 18)
#define FL_WALLOPS			(1 << 19)

#define FL_ZOMBIE			(1 << 31)	/* this item is dead (used by all structures) */

/* Misc */
#define FL_PRIV				(FL_OPERATOR | FL_SERVICE | FL_SERVER)
#define FL_BROADCAST		FL_SERVER	/* used to send chan_* broadcast msg's */

/* Client Channel Flags */
#define CF_OP				(1 << 0)
#define CF_VOICE			(1 << 1)

#define bit_set(x,flag)		 (x)->flags |= (flag)
#define bit_clr(x,flag)		 (x)->flags &= ~(flag)
#define bit_tst(x,flag)		((x)->flags & (flag))

#define is_server(cl)		(bit_tst(cl,FL_SERVER))
#define is_user(cl)			(!bit_tst(cl,FL_SERVER))
#define is_local(cl)		(bit_tst(cl,FL_LOCAL))
#define is_remote(cl)		(!bit_tst(cl,FL_LOCAL))

#define COOKIE_VERIFIED	-1

/* Prototypes */
#ifndef __PLUGIN
	void	init_clients (void);

	client_t *	client_validate	(u_int32_t id);
	client_t *	client_find		(const char *nick);
	client_t *	client_create_remote(client_t *sv, char *name, char *user, char *host);

	void	client_exit		(client_t *cl, const char *comment);

	void	client_read		(client_t *cl);
	void	client_ping		(client_t *cl);
	void	client_register (client_t *cl);
	void	client_userhost (client_t *cl, char *dst, bool hide);
	void	client_userip	(client_t *cl, char *dst, bool hide);

	void	client_accept	(listener_t *listener);
	int		client_select	(listener_t *listener, fd_set *fdp, int maxSock);
	void	client_poll		(listener_t *listener, fd_set *fdp);

	void	client_idle_time(client_t *cl, client_t *target);
	void	client_cookie	(client_t *cl);
	int		client_pong		(client_t *cl, char *arg);

	void	client_set_nick	(client_t *cl, const char *nick);
	int		client_mode_str	(char *buf, char fl, u_int32_t flags);
	void	client_parse_mode_str(client_t *cl, client_t *target, char *mode, bool priv);
#endif /* __PLUGIN */


#ifdef __cplusplus
}
#endif

#endif /* __SRV_CLIENT_H */	 
