/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: ircd_plugins.h,v 1.11 2002/01/29 18:39:39 zarjazz Exp $
*/


#ifndef __SRV_PLUGINS_H
#define __SRV_PLUGINS_H

#ifdef __cplusplus
extern "C" {
#endif


/* Client Interface  */
typedef struct cl_ui_s
{
	/* public variables */
	char *VERSION;
	struct timeval *Now;
	client_t *Srv;
	options_t *Options;
	block_t **Config;
	struct userstats_s *UserStats;
	/* public network functions */
	void		(* cl_send_raw)			(client_t *cl, byte *buffer, int len);
	void		(* cl_send_fmt)			(client_t *cl, const char *fmt, ...);
	void		(* cl_send_numeric)		(client_t *cl, numeric_t numeric, ...);
	void		(* cl_send_names)		(client_t *cl, channel_t *chan, bool show);
	void		(* cl_send_njoin)		(client_t *cl, char *joined[], int j);
	void		(* cl_send_npart)		(client_t *cl, char *parted[], int j);
	void		(* chan_send_fmt)		(channel_t *chan, client_t *ex, u_int32_t fl_cl, u_int32_t fl_index,
											const char *fmt, ...);
	void		(* chan_send_numeric)	(channel_t *chan, client_t *cl, client_t *exclude, numeric_t numeric, ...);
	void		(* all_send_fmt)		(u_int32_t filter, const char *fmt, ...);
	/* public config functions */
	char *		(* config_string)		(const char *block, const char *block_arg, const char *arg);
	/* public client functions */
	void		(* client_exit)			(client_t *cl, const char *comment);
	void		(* client_userhost)		(client_t *cl, char *dst, bool hide);
	void		(* client_userip)		(client_t *cl, char *dst, bool hide);
	void		(* client_register)		(client_t *cl);
	client_t *	(* client_find)			(const char *nick);
	index_t *	(* client_find_invite)	(client_t *cl, channel_t *chan);
	index_t *	(* client_find_chan)	(client_t *cl, channel_t *chan);
	void		(* client_set_nick)		(client_t *cl, const char *nick);
	void		(* client_idle_time)	(client_t *cl, client_t *target);
	void		(* client_cookie)		(client_t *cl);
	int			(* client_pong)			(client_t *cl, char *arg);
	int			(* client_mode_str)		(char *buf, char fl, u_int32_t flags);
	void		(* client_parse_mode_str)(client_t *cl, client_t *target, char *mode, bool priv);
	/* public channel functions */
	void		(* channel_exit)		(index_t *cl_idx, client_t *cl, const char *comment, int type);
	channel_t *	(* channel_find)		(const char *name);
	index_t *	(* channel_find_nick)	(channel_t *chan, const char *nick);
	int			(* channel_mode_str)	(char *buf, char fl, u_int32_t flags, char *key, u_int16_t limit);
	void		(* channel_parse_mode_str)	(client_t *cl, channel_t *chan, char **args, bool priv);
	void		(* channel_list)		(client_t *cl, bool priv, list_options_t *opts);
	channel_t *	(* channel_join)		(channel_t *chan, char *name, client_t *cl, char *userhost);
	/* public misc stuff */
	void		(* invite_remove)		(client_t *cl, index_t *index);
	void		(* ban_remove)			(channel_t *chan, ban_t *ban, char *userhost);
	void		(* ban_add)				(channel_t *chan, client_t *cl, char *ban_str, char *userhost);
	void		(* ircd_terminate)		(void);
	void		(* ircd_rehash)			(client_t *cl);
	int			(* valid_nick_char)		(int ch);
	int			(* valid_chan_char)		(int ch);
	char *		(* sanify_ban)			(char *dst, char *src);
	void		(* debug_memory)		(client_t *cl);
	void		(* show_motd)			(client_t *cl);
	u_int32_t	(* chan_mode)			(char ch);
	u_int32_t	(* user_mode)			(char ch);
	void		(* server_notice)		(const char *msg, const char *arg);
	client_t *	(* server_match)		(const char *str);
} cl_ui_t;


/* Server Interface */
typedef struct sv_ui_s
{
	/* public variables */
	struct timeval *Now;
	client_t *Srv;
	/* public functions */
	void		(* server_exit)		(client_t *sv, const char *comment);
	void		(* server_copy)		(client_t *dst, client_t *src, u_int16_t id);
	void		(* server_register)	(client_t *sv);
	char *		(* server_password) (client_t *sv);
	void		(* server_create)	(client_t *sv, bool lock);
	client_t *	(* server_malloc)	(u_int16_t id);
	client_t *	(* server_find)		(const char *name);
	client_t *	(* server_find_id)	(u_int16_t id);
	void		(* server_notice)	(const char *msg, const char *arg);
	void		(* server_ping)		(client_t *sv);
	int			(* server_pong)		(client_t *sv, char *arg);
	/* public network functions */
	void		(* sv_send_raw)		(client_t *sv, byte *buffer, int len);
	void		(* sv_send_fmt)		(client_t *sv, const char *fmt, ...);
	void		(* sv_send_notice)	(client_t *sv, const char *msg, const char *arg);
	void		(* sv_send_numeric)	(client_t *sv, numeric_t numeric, ...);
	void		(* chan_send_fmt)	(channel_t *chan, client_t *ex, u_int32_t fl_cl, u_int32_t fl_index,
										const char *fmt, ...);
	/* public client functions */
	void		(* client_exit)			(client_t *cl, const char *comment);
	void		(* client_userhost)		(client_t *cl, char *dst, bool hide);
	void		(* client_userip)		(client_t *cl, char *dst, bool hide);
	client_t *	(* client_find)			(const char *nick);
	client_t *	(* client_create_remote)(client_t *sv, char *name, char *user, char *host);
	index_t *	(* client_find_chan)	(client_t *cl, channel_t *chan);
	void		(* client_parse_mode_str)(client_t *cl, client_t *target, char *mode, bool priv);
	/* public channel functions */
	channel_t *	(* channel_create)		(const char *name);
	channel_t *	(* channel_find)		(const char *name);
	index_t *	(* channel_find_nick)	(channel_t *chan, const char *nick);
	void		(* channel_parse_mode_str)	(client_t *cl, channel_t *chan, char **args, bool priv);
	void		(* channel_exit)		(index_t *cl_idx, client_t *cl, const char *comment, int type);
	channel_t *	(* channel_join)		(channel_t *chan, char *name, client_t *cl, char *userhost);
	/* public misc stuff */
	void	(* sync_start)		(client_t *sv, char *prefix, bool request);
	void	(* server_link_add)	(client_t *sv, u_int16_t id);
	void	(* server_link_del)	(client_t *sv, u_int16_t id);
	int		(* valid_nick_char)	(int ch);
	int		(* valid_chan_char)	(int ch);
} sv_ui_t;


/* Globals */
//extern cl_ui_t cl_ui;
extern sv_ui_t sv_ui;


#ifdef __cplusplus
}
#endif

#endif /* __SRV_PLUGINS_H */	 
