/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: ircd_servers.h,v 1.11 2002/01/27 22:34:36 zarjazz Exp $
*/

#ifndef __SRV_SERVERS_H
#define __SRV_SERVERS_H

#ifdef __cplusplus
extern "C" {
#endif


/* Typedefs */
typedef struct
{
	char *motd;
	char *network;
	/* client host settings */
	bool encrypt_hosts;
	bool hide_domain;
	bool hide_ip;
	/* can normal users can set +w */
	bool luser_wallops;
	/* send a notice about user join & quits? */
	bool luser_connect;
	/* sv settings */
	int32_t	max_users;
	int32_t	max_clones;
	int32_t	max_channels;
	int32_t	max_bans;
	int32_t	ping_freq;
	/* reserved names */
	array_t rsv_nicks;
	array_t rsv_chans;
} options_t;

#ifndef __PLUGIN
	/* Prototypes */
	void	init_local (void);

	void	server_create		(client_t *sv, bool lock);
	void	server_create_socket(client_t *sv);

	void	server_accept	(u_int16_t sock);
	void	server_close	(client_t *sv);
	void	server_register	(client_t *sv);
	int		server_connect	(client_t *sv);
	char *	server_password (client_t *sv);
	void	server_copy		(client_t *dst, client_t *src, u_int16_t id);

	void	server_notice	(const char *msg, const char *arg);
	void	server_send_notice (client_t *sv, const char *msg, const char *arg);
	void	server_address	(struct sockaddr_in *sa, const char *address);
	void	server_options	(block_t *root, options_t *opt);
	void	server_set		(client_t *sv, block_t *root, block_t *block);

	void	server_link_add (client_t *sv, u_int16_t id);
	void	server_link_del (client_t *sv, u_int16_t id);

	client_t *	server_malloc	(u_int16_t id);
	client_t *	server_find_id	(u_int16_t id);
	client_t *	server_match	(const char *str);
	client_t *	server_route	(client_t *sv);

	void	sync_start		(client_t *sv, char *prefix, bool request);
	int		parse_server	(client_t *sv);

	u_int16_t	local_server_id	(client_t *cl);

	/* Globals */
	extern client_t *Srv;
	extern listener_t *Remote;
#endif /* __PLUGIN */


#ifdef __cplusplus
}
#endif

#endif /* __SRV_SERVERS_H */	 
