/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: core_sendq.c,v 1.2 2002/01/13 21:33:36 zarjazz Exp $
	----
	Description:

	A thread safe network message que.
*/

#include <sys_include.h>
#include <sys_common.h>
#include <sys_network.h>

#include <core_threads.h>
#include <core_sendq.h>


/*
================
sendq_init
================
*/

void sendq_init	(sendq_t *sendq)
{
	sendq->npackets = 0;
	sendq->nbytes = 0;
	sendq->start = NULL;
	MUTEX_INIT(sendq->mutex);
}

/*
================
sendq_insert
================
*/

void sendq_insert (sendq_t *sendq, sendq_pkt_t *entry)
{
	assert(sendq);
	assert(entry);

	MUTEX_LOCK(sendq->mutex);

	if (sendq->end)
		sendq->end->next = entry;
	else
		sendq->start = entry;
	sendq->end = entry;

	sendq->nbytes += entry->pkt.size;
	sendq->npackets++;

	MUTEX_UNLOCK(sendq->mutex);
}

/*
==============
sendq_remove
--------------
remove 1st (done) packet and return entry
==============
*/

sendq_pkt_t *sendq_remove (sendq_t *sendq)
{
	sendq_pkt_t *entry;

	assert(sendq);

	MUTEX_LOCK(sendq->mutex);

	if (entry = sendq->start)
	{
		if (entry == sendq->end)
			sendq->start = sendq->end = NULL;
		else
			sendq->start = entry->next;

		sendq->nbytes -= entry->pkt.size;
		sendq->npackets--;
	}

	MUTEX_UNLOCK(sendq->mutex);

	return entry;
}
