/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: ircd_commands.c,v 1.6 2002/01/18 20:47:40 zarjazz Exp $
*/

#include <ircd_include.h>

/* ================ */

/* command list */
cmd_entry_t *CMD_ENTRY_LIST;
cmd_entry_t *MSG_ENTRY_LIST;

rwlock_t rwlock_commands;

/*
================
parse stuff
================
*/

static int isseperator (int ch) { return (isspace(ch) || ch == ','); }
static char *skip_space (const char *src) { while (*src &&  isseperator(*src)) src++; return (char *) src; }
static char *skip_token (const char *src) { while (*src && !isseperator(*src)) src++; return (char *) src; }

/*
================
parse_command
----------------
now we could write a whole load of RFC compliant code here,
but personally I really couldn't give a fuck atm :)
================
*/

char *parse_command (int *pargc, char *pargv[], byte *data)
{
	char *str = data, *cmd, *arg;
	int i;

	/* check for prefix */
	if (*str == ':')
	{
		arg = skip_space(++str);
		if (*arg == '\0') return NULL;
		str = skip_token(str);
		if (*str) *str++ = '\0';
		pargv[0] = arg;
	}

	/* extract the command */
	cmd = skip_space(str);
	if (*cmd == '\0') return NULL;
	str = skip_token(str);
	if (*str) *str++ = '\0';

	/* extract all the arguments */
	for (i = 1; i < (MAX_ARGS - 2); i++)
	{
		arg = skip_space(str);
		if (*arg == '\0') break;

		(*pargc)++;
		if (*arg == ':')
		{
			pargv[i] = str_strip(arg + 1);
			break;
		}
		pargv[i] = arg;

		str = skip_token(arg);
		if (*str) *str++ = '\0';
	}

	return cmd;
}

/*
================
cmd_parse
----------------
parse a client message
execute message handler or forward onto remote server
================
*/

void cmd_parse (client_t *cl, byte *data, size_t size)
{
	int pargc = 0;
	byte msg[BUFSIZE];
	cmd_entry_t *cstr;
	char *cmd, *pargv[MAX_ARGS];

	assert(data);

	memcpy(msg,data,size);
	memset(pargv,0,sizeof(pargv));
	cmd = parse_command(&pargc,pargv,msg);
	if (!cmd) return;

	/* execute command */
	for (cstr = is_server(cl) ? MSG_ENTRY_LIST : CMD_ENTRY_LIST; cstr->cmd; cstr++)
	{
		if ( stricmp(cstr->cmd,cmd) ) continue;

		sys_dprintf(2,"Parsed: %s\n",cmd);

		/* set timestamp of last parsed command */
		if (bit_tst(cl,FL_REGISTERED) && bit_tst(cstr,CMD_IDLE)) cl->tv_active = Now.tv_sec;

		/* permission check */
		if (bit_tst(cstr,CMD_ISREG) && !bit_tst(cl,FL_REGISTERED))
			cl_send_numeric(cl, ERR_NOTREGISTERED, cstr->cmd);
		else if (bit_tst(cstr,CMD_ISOPER) && !bit_tst(cl,FL_OPERATOR))
			cl_send_numeric(cl, ERR_NOPRIVILEGES);
		/* call the command function */
		else if (cstr->func)
		{
			int rc;

			/* does this cmd require channel read lock? */
			if (bit_tst(cstr,CMD_WRCHAN)) WRITE_LOCK(rwlock_channels);
			else if (bit_tst(cstr,CMD_RDCHAN)) READ_LOCK(rwlock_channels);
			rc = cstr->func(cl,pargc,pargv);
			if (bit_tst(cstr,CMD_WRCHAN)) WRITE_UNLOCK(rwlock_channels);
			else if (bit_tst(cstr,CMD_RDCHAN)) READ_UNLOCK(rwlock_channels);

			/* if terminal error occured, close the connection */
			if (rc < 0)
			{
				if (is_server(cl))
					server_close(cl);
				else
					client_exit(cl,NULL);
			}
		}
		else
			sys_warn("Unimplemented: %s\n",cstr->cmd);

		return;
	}

	/* send error msg */
	cl_send_numeric(cl, ERR_UNKNOWNCOMMAND, cmd);
}
