/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: net_main.c,v 1.11 2002/01/27 22:34:36 zarjazz Exp $
*/

#define __USE_SERVER_STRUCT
#include <ircd_include.h>

/* ================ */

struct timeval Now;
int Offset = 0;

/*
================
net_loop_parent
================
*/

void net_loop_parent (void)
{
	/* parent thread loop */
	while (!terminate)
	{
		gettimeofday(&Now,NULL);
		Now.tv_sec += Offset;
		msleep(100);
	}
}

/*
================
net_loop_listener
----------------
function called by listener thread (1 per listen port)
================
*/

void net_loop_listener (listener_t *listener)
{
	int rc, maxSock;
	struct timeval timeout;
	fd_set fd, *fdp = &fd;

	assert(listener);

	maxSock = 0;
	FD_ZERO(fdp);
	/* select on listening socket */
	if ( !bit_tst(listener,FL_BUSY) ) SELECT(listener->sock);
	/* select on client sockets */
	maxSock = client_select(listener,fdp,maxSock);
	/* sleep if 0 select size */
	if (!maxSock) { msleep(100); return; }
	/* 1 second timeout */
	timeout.tv_sec	= 1;
	timeout.tv_usec	= 0;
	/* do the select() */
	rc = select(maxSock+1, fdp, NULL, NULL, &timeout);
	if (rc < 0) { sys_warn("child: %s", net_str_error()); return; }
	/* accept() any new connections */
	if ( LISTEN(listener->sock) ) client_accept(listener);
	/* detect data on client sockets */
	client_poll(listener,fdp);
}

/*
================
net_loop_server
----------------
function called by server thread
================
*/

void net_loop_server (client_t *sv)
{
	unsigned sleep = 0;
	int rc;

	while (!terminate && !sv->die)
	{
		if (sleep) { msleep(sleep); sleep = 0; }

		if (bit_tst(sv, FL_BUSY | FL_ZOMBIE))
			{ sleep = 100; continue; }
		else if (bit_tst(sv,FL_CONNECTED))
		{
			int maxSock = 0;
			struct timeval timeout;
			fd_set fd, *fdp = &fd;

			FD_ZERO(fdp);
			/* select on live socket */
			SELECT(sv->sock);

			/* sleep if 0 select size */
			if (!maxSock) { sleep = 100; continue; }
			/* 1 second timeout */
			timeout.tv_sec	= 1;
			timeout.tv_usec	= 0;
			/* do the select() */
			rc = select(maxSock+1, fdp, NULL, NULL, &timeout);
			if (rc < 0) { sys_warn("server: %s", net_str_error()); continue; }
			/* read any data */
			if ( LISTEN(sv->sock) )
			{
				bit_set(sv,FL_BUSY);
				client_read(sv);
			}
		}
		/* start a connection if none present */
		else if (sv->tv_connect < Now.tv_sec)
		{
			/* create socket if required */
			if (!sv->sock)
			{
				server_create_socket(sv);
				sleep = 1000;
			}
			else
			{
				/* try and connect */
				server_notice("Attempting Link", sv->name);
				rc = server_connect(sv);
				if (rc == 0) net_sock_noblock(sv->sock);
			}
		}
	}
}

/*
================
read_timeout
================
*/

int read_timeout (u_int16_t sock, unsigned secs, unsigned usecs)
{
	int rc, maxSock;
	struct timeval timeout;
	fd_set fd, *fdp = &fd;

	maxSock = 0;
	FD_ZERO(fdp);

	/* select on socket */
	SELECT(sock);
	/* set timeout */
	timeout.tv_sec	= secs;
	timeout.tv_usec	= usecs;
	/* do the select() */
	rc = select(maxSock+1, fdp, NULL, NULL, &timeout);
	if (rc <= 0) return -1;
	return 0;
}

/*
================
send_timeout
================
*/

int send_timeout (u_int16_t sock, unsigned secs, unsigned usecs)
{
	int rc, maxSock;
	struct timeval timeout;
	fd_set fd, *fdp = &fd;

	maxSock = 0;
	FD_ZERO(fdp);

	/* select on socket */
	SELECT(sock);
	/* set timeout */
	timeout.tv_sec	= secs;
	timeout.tv_usec	= usecs;
	/* do the select() */
	rc = select(maxSock+1, NULL, fdp, NULL, &timeout);
	if (rc <= 0) return -1;
	return 0;
}
