/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: net_numerics.c,v 1.2 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <ircd_include.h>

/* ================ */

static char *_numeric_table[1000];

static numeric_string_t _numerics[] =
{
	{ RPL_WELCOME_IRC,		":Welcome to the Internet Relay Network %s" },
	{ RPL_WELCOME_HOST,		":Your host is %s, running sircd v%s" },
	{ RPL_WELCOME_SERV,		":This server was create %s at %s" },
	{ RPL_WELCOME_VER,		"%s %s %s" },
	{ RPL_NETWORK_IS,		"NETWORK=%s" },

	{ RPL_TRACELINK,		"Link %s %s %s" },
	{ RPL_TRACECONNECTING,	"Try. %s %s" },
	{ RPL_TRACEHANDSHAKE,	"H.S. %s %s" },
	{ RPL_TRACEUNKNOWN,		"???? %s [%s]" },
	{ RPL_TRACEOPERATOR,	"Oper %s %s" },
	{ RPL_TRACEUSER,		"User %s %s" },
	{ RPL_TRACESERVER,		"Serv %s %uS %uC %s %s!%s@%s" },
	{ RPL_TRACENEWTYPE,		"%s 0 %s" },
	{ RPL_STATSLINKINFO,	"%s %u %s %u %s %u %s" },
	{ RPL_STATSCOMMANDS,	"%s %u" },
	{ RPL_STATSCLINE,		"C %s * %s %hu %s" },
	{ RPL_STATSNLINE,		"N %s * %s %hu %s" },
	{ RPL_STATSILINE,		"I %s * %s %hu %s" },
	{ RPL_STATSKLINE,		"K %s * %s %hu %s" },
	{ RPL_STATSYLINE,		"Y %s %u %u %u" },
	{ RPL_ENDOFSTATS,		"%c :End of /STATS report"},
	{ RPL_UMODEIS,			"%s" },
	{ RPL_STATSLLINE,		"L %s * %s %u" },
	{ RPL_STATSUPTIME,		":Server Up %u days %u:%02d:%02d" },
	{ RPL_STATSOLINE,		"O %s * %s" },
	{ RPL_STATSHLINE,		"H %s * %s" },
	{ RPL_STATSDEBUG,		":%s" },
	{ RPL_LUSERCLIENT,		":There are %u users and %u invisible on %u servers" },
	{ RPL_LUSEROP,			"%u :operator(s) online" },
	{ RPL_LUSERUNKNOWN,		"%u :unknown connection(s)" },
	{ RPL_LUSERCHANNELS,	"%u :channels formed" },
	{ RPL_LUSERME,			":I have %u clients and %u servers" },
	{ RPL_ADMINME,			":Administrative info for %s" },
	{ RPL_ADMINLOC1,		":%s" },
	{ RPL_ADMINLOC2,		":%s" },
	{ RPL_ADMINEMAIL,		":%s" },
	{ RPL_TRACELOG,			"File %s %s" },

	{ RPL_NONE,				NULL },
	{ RPL_AWAY,				"%s :%s" },
	{ RPL_USERHOST,			":%s%s=%c%s" },
	{ RPL_ISON,				":%s" },
	{ RPL_UNAWAY,			":You are no longer marked as being away" },
	{ RPL_NOWAWAY,			":You have been marked as being away" },
	{ RPL_WHOISUSER,		"%s %s %s * :%s" },
	{ RPL_WHOISSERVER,		"%s %s :%s" },
	{ RPL_WHOISOPERATOR,	"%s :is an IRC operator" },
	{ RPL_WHOWASUSER,		"%s %s %s * :%s" },
	{ RPL_ENDOFWHO,			"%s :End of /WHO list" },
	{ RPL_WHOISIDLE,		"%s %u %u :seconds idle, signon time" },
	{ RPL_ENDOFWHOIS,		"%s :End of /WHOIS list" },
	{ RPL_WHOISCHANNELS,	"%s :%s" },
	{ RPL_LISTSTART,		"Channel :Users  Name" },
	{ RPL_LIST,				"%s %u :%s" },
	{ RPL_LISTEND,			":End of /LIST" },
	{ RPL_CHANNELMODEIS,	"%s %s" },
	{ RPL_NOTOPIC,			"%s :No topic is set" },
	{ RPL_TOPIC,			"%s :%s" },
	{ RPL_TOPICSTAMP,		"%s %s %u" },
	{ RPL_INVITING,			"%s %s" },
	{ RPL_SUMMONING,		"%s :Summoning user to IRC" },
	{ RPL_VERSION,			"%s.%s %s :%s" },
	{ RPL_WHOREPLY,			"%s %s %s %s %s %s :%u %s" },
	{ RPL_NAMREPLY,			"%c %s :" },
	{ RPL_LINKS,			"%s %s :%u %s" },
	{ RPL_ENDOFLINKS,		"%s :End of /LINKS list" },
	{ RPL_ENDOFNAMES,		"%s :End of /NAMES list" },
	{ RPL_BANLIST,			"%s %s %s %u" },
	{ RPL_ENDOFBANLIST,		"%s :End of channel ban list" },
	{ RPL_ENDOFWHOWAS,		"%s :End of WHOWAS" },
	{ RPL_INFO,				":%s" },
	{ RPL_MOTD,				":- %s" },
	{ RPL_ENDOFINFO,		":End of /INFO list" },
	{ RPL_MOTDSTART,		":- %s Message of the day - " },
	{ RPL_ENDOFMOTD,		":End of /MOTD command" },
	{ RPL_YOUREOPER,		":You are now an IRC operator" },
	{ RPL_REHASHING,		"%s :Rehashing" },
	{ RPL_TIME,				"%u %u :%s" },
	{ RPL_USERSSTART,		":UserID   Terminal  Host" },
	{ RPL_USERS,			":%-8s %-9s %-8s" },
	{ RPL_ENDOFUSERS,		":End of users" },
	{ RPL_NOUSERS,			":Nobody logged in" },

	{ ERR_NOSUCHNICK,		"%s :No such nick" },
	{ ERR_NOSUCHSERVER,		"%s :No such server" },
	{ ERR_NOSUCHCHANNEL,	"%s :No such channel" },
	{ ERR_CANNOTSENDTOCHAN,	"%s :Cannot send to channel" },
	{ ERR_TOOMANYCHANNELS,	"%s :You have joined too many channels" },
	{ ERR_WASNOSUCHNICK,	"%s :There was no such nickname" },
	{ ERR_TOOMANYTARGETS,	"%s :Duplicate recipients. No message delivered" },
	{ ERR_NOORIGIN,			":No origin specified" },
	{ ERR_NORECIPIENT,		":No recipient given (%s)" },
	{ ERR_NOTEXTTOSEND,		":No text to send" },
	{ ERR_NOTOPLEVEL,		"%s :No toplevel domain specified" },
	{ ERR_WILDTOPLEVEL,		"%s :Wildcard in toplevel domain" },
	{ ERR_UNKNOWNCOMMAND,	"%s :Unknown command" },
	{ ERR_NOMOTD,			":No MOTD available" },
	{ ERR_NOADMININFO,		"%s :No administrative info available" },
	{ ERR_FILEERROR,		":File error doing %s on %s" },
	{ ERR_NONICKNAMEGIVEN,	":No nickname given" },
	{ ERR_ERRONEUSNICKNAME,	"%s :Erroneus nickname" },
	{ ERR_NICKNAMEINUSE,	"%s :Nickname is already in use" },
	{ ERR_SERVICENICK,		"%s :Nickname registered to Network Service" },
	{ ERR_NICKCOLLISION,	"%s :Nickname collision KILL" },
	{ ERR_USERNOTINCHANNEL,	"%s %s :They aren't on that channel" },
	{ ERR_NOTONCHANNEL,		"%s :You're not on that channel" },
	{ ERR_USERONCHANNEL,	"%s %s :is already on channel" },
	{ ERR_NOLOGIN,			"%s :User not logged in" },
	{ ERR_SUMMONDISABLED,	":SUMMON has been disabled" },
	{ ERR_USERSDISABLED,	":USERS has been disabled" },
	{ ERR_NOTREGISTERED,	"* %s :You have not registered" },
	{ ERR_NEEDMOREPARAMS,	"%s :Not enough parameters" },
	{ ERR_ALREADYREGISTRED,	":You may not reregister" },
	{ ERR_NOPERMFORHOST,	":Your host isn't among the privileged" },
	{ ERR_PASSWDMISMATCH,	":Password incorrect" },
	{ ERR_YOUREBANNEDCREEP,	":You are banned from this server" },
	{ ERR_KEYSET,			"%s :Channel key already set" },
	{ ERR_CHANNELISFULL,	"%s :Cannot join channel (+l)" },
	{ ERR_UNKNOWNMODE,		"%c :is unknown mode char to me" },
	{ ERR_INVITEONLYCHAN,	"%s :Cannot join channel (+i)" },
	{ ERR_BANNEDFROMCHAN,	"%s :Cannot join channel (+b)" },
	{ ERR_BADCHANNELKEY,	"%s :Cannot join channel (+k)" },
	{ ERR_SSLONLYCHAN,		"%s :Cannot join channel (+x)" },
	{ ERR_BANLISTFULL,		"%s %s :Channel ban list is full" },
	{ ERR_NOPRIVILEGES,		":Permission Denied- You're not an IRC operator" },
	{ ERR_CHANOPRIVSNEEDED,	"%s :You're not a channel operator" },
	{ ERR_CANTKILLSERVER,	":You cant kill a server!" },
	{ ERR_NOOPERHOST,		":No O-lines for your host" },

	{ ERR_UMODEUNKNOWNFLAG,	":Unknown MODE flag '%c'" },
	{ ERR_USERSDONTMATCH,	":Cant change mode for other users" },
	/* Terminiator */
	{ RPL_TERMINATOR,		NULL }
};

/*
================
init_numerics
================
*/

void init_numerics (void)
{
	numeric_string_t *ptr;

	memset(_numeric_table,0,sizeof(_numeric_table));
	for (ptr = &_numerics[0]; ptr->id; ptr++)
		_numeric_table[ptr->id] = ptr->fmt;
}

/*
================
numeric_string
================
*/

const char *numeric_string (numeric_t numeric)
{
	return _numeric_table[(int) numeric];
}
