/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Read the LICENSE file for more details
	----
	$Id: net_scan.c,v 1.3 2002/01/13 21:33:36 zarjazz Exp $
*/

#include <ircd_include.h>

/* ================ */

/*
================
check_socks4
================
*/

int check_socks4 (client_t *cl)
{
	byte request[16], response[64];
	sys_pkt_t pkt;
	struct sockaddr_in sa, cl_sa;
	u_int16_t sock = 0;
	int rc;

	/* create scan socket */
	net_sockaddr(&sa,NULL,0);
	rc = net_sock_tcp(&sa);
	if (rc < 0) { sys_warn("socks scan %s","socket"); return -1; }
	sock = rc;
	/* try to connect to client */
	memcpy(&cl_sa,&cl->sa,sizeof(cl_sa));
	cl_sa.sin_port = htons(1080);
	rc = connect(sock,(struct sockaddr *) &cl_sa,sizeof(cl_sa));
	if (rc < 0) { sys_dprintf(3,"socks scan: %s ",net_str_error()); goto clean; }
	/* initialize request */
	request[0] = 4;
	request[1] = 1;
	memcpy(request + 2, &sa.sin_port, 2);
	memcpy(request + 4, &sa.sin_addr, 4);
	request[8] = 0;

	/* send request */
	pkt.data = request;
	pkt.size = 9;
	rc = net_send(&pkt,sock);
	if (rc < 0) goto err;
	/* read response */
	pkt.data = response;
	rc = net_read(&pkt,sock,(struct sockaddr *) &cl_sa,sizeof(response));
	if (rc < 0) goto err;
	/* check response */
/*	if (response[0] != 4) goto clean;*/	/* many socks implementations set that wrong */
	if (response[1] != 90) goto clean;

	/* scan is positive */
	net_sock_close(sock);
	return 1;
clean:
	net_sock_close(sock);
	return 0;
err:
	net_sock_close(sock);
	return -1;
}

/*
================
check_socks5
================
*/

int check_socks5 (client_t *cl)
{
	byte request[16], response[64];
	sys_pkt_t pkt;
	struct sockaddr_in sa, cl_sa;
	u_int16_t sock = 0;
	int rc;

	/* create scan socket */
	net_sockaddr(&sa,NULL,0);
	rc = net_sock_tcp(&sa);
	if (rc < 0) { sys_warn("socks scan %s","socket"); return -1; }
	sock = rc;
	/* try to connect to client */
	memcpy(&cl_sa,&cl->sa,sizeof(cl_sa));
	cl_sa.sin_port = htons(1080);
	rc = connect(sock,(struct sockaddr *) &cl_sa,sizeof(cl_sa));
	if (rc < 0)
	{
		net_sock_close(sock);
		sys_dprintf(3,"socks scan: %s ",net_str_error());
		return 0;
	}
	/* initialize request */
	request[0] = 5;
	request[1] = 1;
	request[2] = 0;
	request[3] = 1;
	memcpy(request + 4, &sa.sin_addr, 4);
	memcpy(request + 8, &sa.sin_port, 2);

	/* send 1st request */
	pkt.data = request;
	pkt.size = 3;
	rc = net_send(&pkt,sock);
	if (rc < 0) goto err;
	/* read 1st response */
	pkt.data = response;
	rc = net_read(&pkt,sock,(struct sockaddr *) &cl_sa,sizeof(response));
	if (rc < 0) goto err;
	/* check 1st response */
	if (response[0] != 5) goto clean;
	if (response[1] != 0) goto clean;

	/* send 2nd request */
	pkt.data = request;
	pkt.size = 10;
	rc = net_send(&pkt,sock);
	if (rc < 0) goto err;
	/* read 2nd response */
	pkt.data = response;
	rc = net_read(&pkt,sock,(struct sockaddr *) &cl_sa,sizeof(response));
	if (rc < 0) goto err;
	/* check 2nd response */
	if (response[0] != 5) goto clean;
	if (response[1] != 0) goto clean;

	/* scan is positive */
	net_sock_close(sock);
	return 1;
clean:
	net_sock_close(sock);
	return 0;
err:
	net_sock_close(sock);
	return -1;
}
