/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/
	----
	$Id: sys_common.h,v 1.4 2001/11/27 21:58:33 zarjazz Exp $
*/

#ifndef __SYS_COMMON_H
#define __SYS_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

/* Color Terminal Settings */

DLL_API	char s_red[];
DLL_API	char s_green[];
DLL_API	char s_yellow[];
DLL_API	char s_purple[];
DLL_API	char s_cyan[];
DLL_API	char s_normal[];

DLL_API	void	sys_color	(char *s);

/* Prototypes */
DLL_API	void	sys_cleanup_register	(cleanup_func_t func);
DLL_API	void	sys_cleanup				(void);

DLL_API	int		sys_debug_get	(void);
DLL_API	void	sys_debug_set	(int lvl);

DLL_API			char *	sys_str_error	(void);
DLL_API	const	char *	sys_str_time	(void);

DLL_API	void	sys_error		(const char *msg, ...);
DLL_API	void	sys_warn		(const char *msg, ...);
DLL_API	void	sys_dprintf		(int lvl, const char *msg, ...);
DLL_API	void	sys_printf		(const char *msg, ...);
DLL_API	void	sys_print_raw	(FILE *f, byte *d, size_t size);

DLL_API	void	sys_log_init	(int mode, const char *fname);
DLL_API	void	sys_log_cleanup	(void);
DLL_API	FILE *	sys_log_get		(void);

DLL_API	byte *	sys_file_read	(const char *fname, size_t *size);

#ifdef WINVER
DLL_API	void	gettimeofday	(struct timeval *tv, void *tz);
#endif
DLL_API	int32_t tv_diff			(struct timeval *start, struct timeval *end);

/* Memory (De)Allocation */

DLL_API	void *	sys_malloc	(size_t size);
DLL_API	void *	sys_realloc	(void *ptr, size_t size);
DLL_API	void	sys_free	(void *ptr);

#ifdef WINVER

DLL_API	void *	win_malloc	(size_t size);
DLL_API	void *	win_realloc (void *ptr, size_t size);
DLL_API	void	win_free	(void *ptr);

#else

#define	win_malloc	sys_malloc
#define	win_realloc sys_realloc
#define	win_free	sys_free

#endif /* WINVER */

#define bit_set(x,flag)		 (x)->flags |= (flag)
#define bit_clr(x,flag)		 (x)->flags &= ~(flag)
#define bit_tst(x,flag)		((x)->flags & (flag))


#ifdef __cplusplus
}
#endif

#endif /* __COMMON_H */
