/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/
	----
	$Id: sys_config.h,v 1.6 2001/11/27 21:58:33 zarjazz Exp $
*/

#ifndef __SYS_CONFIG_H
#define __SYS_CONFIG_H

#ifdef __cplusplus
extern "C" {
#endif

/* Basic Config Typedefs */
typedef struct config_s
{
	/* List Structures */
	struct config_s *next;
	struct config_s *prev;
	/* Item Specific Data */
	char *item;
	char *value;
} config_t;

/* Block Config Typedefs */
typedef struct
{
	char *arg_name;
	char *arg_value;
} block_arg_t;

typedef struct block_s
{
	/* List Structures */
	struct block_s *next;
	struct block_s *prev;
	/* Item Specific Data */
	llist_t	sub_blocks;
	char *block_name;
	char *block_value;
	block_arg_t **block_args;
	int	nargs;
} block_t;

typedef struct
{
	char *value;
	u_int16_t flags;
} conf_entry_t;

typedef struct
{
	char *name;
	conf_entry_t *entries;
} conf_block_t;

/* Defines */

#define		CF_REQUIRED		(1 << 0)	/* Must be included in configuration */
#define		CF_MULTIREF		(1 << 1)	/* Allow multiple references per block */
#define		CF_ARGUMENT		(1 << 2)	/* Argument Required */

/* Prototypes */

DLL_API	void		cleanup_config	(void);
DLL_API	void		init_config		(const char *fname);
DLL_API	void		config_print	(void);
DLL_API	char *		config_value	(const char *arg);
DLL_API	config_t *	config_get		(void);

DLL_API block_t *	init_block	(const char *fname);
DLL_API void		cleanup_block	(block_t *block);
DLL_API void		print_block	(block_t *block, int lvl);

DLL_API	void		block_syntax_check	(block_t *block, conf_block_t **block_table, int block_table_size);
DLL_API void		argument_range		(block_arg_t *range, array_t *array);
DLL_API void		argument_list		(block_arg_t *range, array_t *array);

DLL_API	block_t *		blocks_by_name		(block_t *block, block_t *previous, const char *name);
DLL_API	block_arg_t *	arguments_by_name	(block_t *block, block_arg_t *previous, const char *name);
DLL_API	char *			argument_by_name	(block_t *block, const char *name);


#ifdef __cplusplus
}
#endif

#endif /* __SYS_CONFIG_H */
