/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/
	----
	$Id: sys_include.h,v 1.6 2001/11/30 16:26:19 zarjazz Exp $
*/

#ifndef __SYS_INCLUDE_H
#define __SYS_INCLUDE_H

#if defined(__GUNC__) || defined(__linux__)
#	define _GNU_SOURCE
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <errno.h>
#include <signal.h>
#include <assert.h>
#include <sys/types.h>
#include <time.h>

/* OS specific header files */
#ifdef WINVER
#	define WIN32_LEAN_AND_MEAN
#	include <windows.h>
#	include <windowsx.h>
#elif __linux__
#	include <sys/time.h>
#endif /* WINVER */

#undef DLL_API

#ifdef SYSCORE_EXPORTS
#	define DLL_API __declspec(dllexport)
#elif WINVER
#	define DLL_API __declspec(dllimport)
#else
#	define DLL_API extern
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* Endian Stuff */

#ifndef BYTE_ORDER

#if (BSD >= 199103)
	#include <machine/endian.h>
#elif defined(linux)
	#include <endian.h>
#else
	#define LITTLE_ENDIAN   1234    /* least-significant byte first (vax, pc) */
	#define BIG_ENDIAN      4321    /* most-significant byte first (IBM, net) */
	#define PDP_ENDIAN      3412    /* LSB first in word, MSW first in long (pdp) */

	#if defined(vax) || defined(ns32000) || defined(sun386) || defined(i386) || \
		defined(__ia64) || defined (WINVER) || \
		defined(MIPSEL) || defined(_MIPSEL) || defined(BIT_ZERO_ON_RIGHT) || \
		defined(__alpha__) || defined(__alpha)

		#define BYTE_ORDER      LITTLE_ENDIAN
	#endif

	#if defined(sel) || defined(pyr) || defined(mc68000) || defined(sparc) || \
		defined(is68k) || defined(tahoe) || defined(ibm032) || defined(ibm370) || \
		defined(MIPSEB) || defined(_MIPSEB) || defined(_IBMR2) || defined(DGUX) ||\
		defined(apollo) || defined(__convex__) || defined(_CRAY) || \
		defined(__hppa) || defined(__hp9000) || \
		defined(__hp9000s300) || defined(__hp9000s700) || \
		defined(BIT_ZERO_ON_LEFT) || defined(m68k)

		#define BYTE_ORDER      BIG_ENDIAN
	#endif
#endif

#endif /* BYTE_ORDER */

#if !defined(BYTE_ORDER) || (BYTE_ORDER != BIG_ENDIAN && BYTE_ORDER != LITTLE_ENDIAN && BYTE_ORDER != PDP_ENDIAN)
	/*
	 * you must determine what the correct bit order is for
	 * your compiler - the next line is an intentional error
	 * which will force your compiles to bomb until you fix
	 * the above macros.
	 */
	#error "Undefined or invalid BYTE_ORDER"
#endif

/* Typedefs */

#if defined(WINVER)
	typedef char    int8_t;
	typedef short   int16_t;
	typedef long    int32_t;
	typedef __int64 int64_t;

	typedef unsigned char    u_int8_t;
	typedef unsigned short   u_int16_t;
	typedef unsigned long    u_int32_t;
	typedef unsigned __int64 u_int64_t;
#elif defined(__sun__)
	#define u_int8_t uint8_t
	#define u_int16_t uint16_t
	#define u_int32_t uint32_t
	#define u_int64_t uint64_t
#endif

#ifdef WINVER
	#define msleep(x)	Sleep((x))
	#define vsnprintf	_vsnprintf
	#define alloca		_alloca
#else
	#define stricmp strcasecmp
	#define strnicmp strncasecmp
	#define msleep(x) usleep(1000 * (x))
#endif

typedef enum { false, true } bool;
typedef unsigned char byte;

typedef void (*cleanup_func_t) (void);


#ifdef __cplusplus
}
#endif

#endif /* __SYS_INCLUDE_H */
