/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/
	----
	$Id: sys_network.h,v 1.6 2001/11/27 17:32:03 zarjazz Exp $
*/

#ifndef __NETWORK_H
#define __NETWORK_H

#ifdef WINVER
	/* Win32 network files */
	#include <winsock2.h>
	#include <ws2tcpip.h>
	#include <windows.h>
#else
	/* Unix header files */
	#include <sys/socket.h>
	#include <sys/types.h>
	#include <netinet/in.h>
	#include <netinet/tcp.h>
	#include <arpa/inet.h>
	#include <netdb.h>

	#include <fcntl.h>
	#include <unistd.h>
	#include <errno.h>
#endif /* WINVER */

#ifdef __cplusplus
extern "C" {
#endif


/* Defines */

#ifndef MAXHOSTNAMELEN
	#define MAXHOSTNAMELEN 256
#endif

#ifndef INADDR_NONE
	#define INADDR_NONE ((u_int32_t) 0xffffffff)
#endif

/* Typedefs */

typedef struct sys_pkt_s
{
	int32_t size;
	byte *data;
} sys_pkt_t;

/* Prototypes */

DLL_API	void	init_net		(void);
DLL_API	void	cleanup_net		(void);

DLL_API	char *	net_str_error	(void);
DLL_API	char *	net_str_sockaddr(char *s, struct sockaddr_in *sa);
DLL_API	char *	net_str_local	(char *s);

DLL_API	int		net_cmp_sockaddr(struct sockaddr_in *a, struct sockaddr_in *b);

DLL_API	bool	net_resolve		(const char *host_name, struct in_addr *addr);
DLL_API	bool	net_sockaddr	(struct sockaddr_in *sa, const char *host_name, u_int16_t port);

DLL_API	int		net_sock_noblock(u_int16_t sock);
DLL_API	int		net_sock_nodelay(u_int16_t sock);
DLL_API	int		net_sock_buffer	(u_int16_t sock, unsigned int size);

DLL_API	int		net_sock_udp	(struct sockaddr_in *name);
DLL_API	int		net_sock_tcp	(struct sockaddr_in *name);
DLL_API	void	net_sock_close	(u_int16_t sock);

DLL_API void net_dprintf(sys_pkt_t *pkt, char *str, struct sockaddr *name);
DLL_API	int	net_send	(sys_pkt_t *pkt, u_int16_t sock);
DLL_API	int	net_sendto	(sys_pkt_t *pkt, u_int16_t sock, struct sockaddr *dst);
DLL_API	int net_read	(sys_pkt_t *pkt, u_int16_t sock, struct sockaddr *src, size_t len);


#ifdef __cplusplus
}
#endif

#endif /* __NETWORK_H */

