/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Please read the LICENSE file for more details
	----
	$Id: sys_array.c,v 1.4 2001/08/09 10:45:50 zarjazz Exp $
*/

#include <sys_include.h>
#include <sys_common.h>
#include <sys_array.h>

/* ================ */

array_t *array_init (array_t *array, u_int32_t block_size)
{
	if (array)
		memset(array,0,sizeof(*array));
	else
		array = sys_malloc(sizeof(*array));
	if (array) array->nblk = block_size;
	return array;
}

void array_add_item (array_t *array, void *item, size_t item_len)
{
	assert(array);
	/* increase array size if required */
	if ((array->nitems % array->nblk) == 0)
	{
		array->ndim += array->nblk;
		array->list = sys_realloc(array->list, array->ndim * item_len);
	}
	/* add item to array */
	memcpy(array->list + (array->nitems * item_len), item, item_len);
	array->nitems++;
}

void array_del_item (array_t *array, void *item, size_t item_len)
{
	int i;

	assert(array);

	if (!array->nitems) return;

	for (i = 0; i < array->nitems; i++) if ( !memcmp(array->list + (item_len * i), item, item_len) )
	{
		int n;
		/* remove item from array */
		array->nitems--;
		n = array->nitems - i;
		if (n > 0) memcpy(array->list + (item_len * i), array->list + (item_len * n), item_len);
		/* decrease array size if required */
		if ((array->nitems % array->nblk) == 0)
		{
			array->ndim -= array->nblk;
			if (array->nitems == 0)
			{
				sys_free(array->list);
				array->list = NULL;
			}
			else
				array->list = sys_realloc(array->list, item_len * array->ndim);
		}
		return;
	}
}
