/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Please read the LICENSE file for more details
	----
	$Id: sys_config_basic.c,v 1.2 2001/11/30 16:26:19 zarjazz Exp $
*/

#include <sys_include.h>
#include <sys_common.h>
#include <sys_string.h>
#include <sys_array.h>
#include <sys_list.h>
#include <sys_config.h>

/* Locals */

static llist_t _conf;

/*
================
config_cleanup
================
*/

void cleanup_config (void)
{
	config_t *cf, *next;

	for (cf = (config_t *) _conf.item_start; cf; cf = next)
	{
		next = cf->next;
		ll_del_item(&_conf,cf);
		sys_free(cf->value);
		sys_free(cf->item);
		sys_free(cf);
	}
}

/*
================
config_init
----------------
parse a configuration file in "arg = val" format
================
*/

void init_config (const char *fname)
{
	FILE *file;

	char line[128];

	/* Reset the current list */
	memset(&_conf,0,sizeof(_conf));

	/* Open the config file */
	file = fopen(fname, "r");
	if (!file) { sys_error("Unable to open file: %s",fname); return; }

	/* Parse the config */
	while ( fgets(line, sizeof(line), file) )
	{
		char *s1, *s2;
		config_t *cf;

		/* strip the line of white spaces */
		s1 = str_strip(line);
		/* skip emtpy lines and comments */
		if ( *s1 == '\0' || *s1 == '#' || !strncmp(s1,"//",2) ) continue;

		/* is this a valid line? */
		if ( !(s2 = strchr(s1,'=')) ) continue;
		*s2++ = '\0';

		/* parse the 'item = value' section */
		s1 = str_strip(s1);
		s2 = str_strip(s2);

		sys_dprintf(2,"conf: %s = %s\n",s1,s2);

		/* add the item / value to the list */
		cf = sys_malloc(sizeof(*cf));
		if (!cf) return;

		cf->item  = str_dup(s1);
		cf->value = str_dup(s2);

		ll_add_end(&_conf,cf);
	}

	fclose(file);
}

/*
================
config_value
================
*/

char *config_value (const char *arg)
{
	config_t *cf;

	for (cf = (config_t *) _conf.item_start; cf; cf = cf->next)
		if ( !stricmp(cf->item,arg) )
			return cf->value;

	return NULL;
}

/*
===============
config_print
===============
*/

void config_print (void)
{
	config_t *cf;

	sys_printf("Configuration\n");

	for (cf = (config_t *) _conf.item_start; cf; cf = cf->next)
		sys_printf("  %s = %s\n",
		cf->item  ? cf->item  : "??",
		cf->value ? cf->value : "??");
}

/*
================
config_get
================
*/

config_t *config_get (void) { return (config_t *) _conf.item_start; }
