/*
	Author	: Zarjazz
	Email	: zarjazz@sircd.org
	WebPage	: http://www.sircd.org/

	Copyright (C) 2001,2002  Vincent Sweeney
	----
	Please read the LICENSE file for more details
	----
	$Id: sys_list.c,v 1.3 2001/05/21 16:59:12 zarjazz Exp $
	----
	Description:

	A simple 'C' "Double Linked List" implementation.

	Usage:

	Declare a list and define the type of data you want to list

		typedef struct my_item_s
		{
			// Required for the list code (order is important)
			struct my_item_s	*next;
			struct my_item_s	*prev;
			// Item specific data
			int		i;
			char	*txt;
				.
				.
		} my_item_t;

		llist_t		list;
		my_item_t	item;

	Then use the list macros, as shown below, to simplify the type-casting

		ll_add_??? (&list, &item);
		ll_del_item(&list, &item);
*/

#include <sys_include.h>
#include <sys_list.h>

/*
================
ll_add_start
================
*/

void _ll_add_start (llist_t *llist, ll_item_t *item)
{
	assert(llist);
	assert(item);

	if (!llist->nitems) llist->item_end = item;
	if (item->next = llist->item_start) item->next->prev = item;
	llist->item_start = item;
	llist->nitems++;
}

/*
================
ll_add_end
================
*/

void _ll_add_end (llist_t *llist, ll_item_t *item)
{
	assert(llist);
	assert(item);

	if (!llist->nitems)
		llist->item_start = item;
	else
	{
		llist->item_end->next = item;
		item->prev = llist->item_end;
	}
	llist->item_end = item;
	llist->nitems++;
}

/*
==============
ll_del_item

takes a pointer to a llist and a link-item
and removes the pointer from the list.
==============
*/

void _ll_del_item (llist_t *llist, ll_item_t *item)
{
	assert(llist);
	assert(item);

	/* We can't delete from an empty list */
	if (!llist->nitems || !llist->item_start) return;

	if (item == llist->item_start)		/* First Item */
	{
		if (llist->item_start = item->next)
			item->next->prev = NULL;
	}
	else if (item == llist->item_end)	/* Last Item */
	{
		if (llist->item_end = item->prev)
			item->prev->next = NULL;
	}
	else
	{
		item->prev->next = item->next;
		item->next->prev = item->prev;
	}

	if (--llist->nitems == 0) llist->item_start = llist->item_end = (void *) 0; /* NULL */
}
